/***************************************************************************** 
 *                                                                           * 
 *          UNURAN -- Universal Non-Uniform Random number generator          * 
 *                                                                           * 
 *****************************************************************************/
                                                                               
/*---------------------------------------------------------------------------*/
/*  #define DEBUG 1 */                                                         
/*---------------------------------------------------------------------------*/
#include "test_with_Mathematica.h"                                           
/*---------------------------------------------------------------------------*/
                                                                               
int main()                                                                     
{                                                                              
   double fparams[UNUR_DISTR_MAXPARAMS];  /* parameters of distribution      */
   UNUR_DISTR *distr;                     /* distribution object             */
   FILE *UNURANLOG;                       /* unuran log file                 */
   FILE *TESTLOG;                         /* test log file                   */
   int n_failed = 0;                      /* number of failed tests          */
                                                                               
   /* open log file for unuran and set output stream for unuran messages */    
   UNURANLOG = fopen( "t_Mathematica_unuran.log","w" );                    
   if (UNURANLOG == NULL) exit (-1);                                           
   unur_set_stream( UNURANLOG );                                               
                                                                               
   /* open log file for testing */                                             
   TESTLOG = fopen( "t_Mathematica_test.log","w" );                        
   if (TESTLOG == NULL) exit (-1);                                             
                                                                               
   /* write header into log file */                                            
   {                                                                           
      time_t started;                                                          
      fprintf(TESTLOG,"\nUNURAN - Universal Non-Uniform RANdom number generator\n\n");
      if (time( &started ) != -1)                                              
         fprintf(TESTLOG,"%s",ctime(&started));                              
      fprintf(TESTLOG,"\n======================================================\n\n");
      fprintf(TESTLOG,"(Search for string \"data\" to find new section.)\n\n");
   }                                                                           
                                                                               
   fparams[0] = 10.;
   fparams[1] = 100.;
   distr = unur_distr_beta(fparams,2);
   if (test_cdf_pdf( TESTLOG, distr,"t_distr_beta.data" ) == 0)
      n_failed++;
   unur_distr_free(distr);

   fparams[0] = 100.;
   fparams[1] = 100.;
   distr = unur_distr_cauchy(fparams,2);
   if (test_cdf_pdf( TESTLOG, distr,"t_distr_cauchy.data" ) == 0)
      n_failed++;
   unur_distr_free(distr);

   fparams[0] = 100.;
   distr = unur_distr_chi(fparams,1);
   if (test_cdf_pdf( TESTLOG, distr,"t_distr_chi.data" ) == 0)
      n_failed++;
   unur_distr_free(distr);

   fparams[0] = 100.;
   distr = unur_distr_chisquare(fparams,1);
   if (test_cdf_pdf( TESTLOG, distr,"t_distr_chisquare.data" ) == 0)
      n_failed++;
   unur_distr_free(distr);

   fparams[0] = 100.;
   distr = unur_distr_exponential(fparams,1);
   if (test_cdf_pdf( TESTLOG, distr,"t_distr_exponential.data" ) == 0)
      n_failed++;
   unur_distr_free(distr);

   fparams[0] = 100.;
   fparams[1] = 100.;
   distr = unur_distr_extremeI(fparams,2);
   if (test_cdf_pdf( TESTLOG, distr,"t_distr_extremeI.data" ) == 0)
      n_failed++;
   unur_distr_free(distr);

   fparams[0] = 100.;
   fparams[1] = 100.;
   fparams[2] = 100.;
   distr = unur_distr_extremeII(fparams,3);
   if (test_cdf_pdf( TESTLOG, distr,"t_distr_extremeII.data" ) == 0)
      n_failed++;
   unur_distr_free(distr);

   fparams[0] = 10.;
   fparams[1] = 100.;
   distr = unur_distr_gamma(fparams,2);
   if (test_cdf_pdf( TESTLOG, distr,"t_distr_gamma.data" ) == 0)
      n_failed++;
   unur_distr_free(distr);

   fparams[0] = 100.;
   fparams[1] = 100.;
   distr = unur_distr_laplace(fparams,2);
   if (test_cdf_pdf( TESTLOG, distr,"t_distr_laplace.data" ) == 0)
      n_failed++;
   unur_distr_free(distr);

   fparams[0] = 100.;
   fparams[1] = 100.;
   distr = unur_distr_lomax(fparams,2);
   if (test_cdf_pdf( TESTLOG, distr,"t_distr_lomax.data" ) == 0)
      n_failed++;
   unur_distr_free(distr);

   fparams[0] = 100.;
   fparams[1] = 100.;
   distr = unur_distr_logistic(fparams,2);
   if (test_cdf_pdf( TESTLOG, distr,"t_distr_logistic.data" ) == 0)
      n_failed++;
   unur_distr_free(distr);

   fparams[0] = 100.;
   fparams[1] = 100.;
   distr = unur_distr_normal(fparams,2);
   if (test_cdf_pdf( TESTLOG, distr,"t_distr_normal.data" ) == 0)
      n_failed++;
   unur_distr_free(distr);

   fparams[0] = 100.;
   fparams[1] = 100.;
   distr = unur_distr_pareto(fparams,2);
   if (test_cdf_pdf( TESTLOG, distr,"t_distr_pareto.data" ) == 0)
      n_failed++;
   unur_distr_free(distr);

   fparams[0] = 3.;
   distr = unur_distr_powerexponential(fparams,1);
   if (test_cdf_pdf( TESTLOG, distr,"t_distr_powerexponential.data" ) == 0)
      n_failed++;
   unur_distr_free(distr);

   fparams[0] = 100.;
   distr = unur_distr_rayleigh(fparams,1);
   if (test_cdf_pdf( TESTLOG, distr,"t_distr_rayleigh.data" ) == 0)
      n_failed++;
   unur_distr_free(distr);

   fparams[0] = 100.;
   distr = unur_distr_student(fparams,1);
   if (test_cdf_pdf( TESTLOG, distr,"t_distr_student.data" ) == 0)
      n_failed++;
   unur_distr_free(distr);

   fparams[0] = 1.;
   distr = unur_distr_triangular(fparams,1);
   if (test_cdf_pdf( TESTLOG, distr,"t_distr_triangular.data" ) == 0)
      n_failed++;
   unur_distr_free(distr);

   fparams[0] = 1.;
   fparams[1] = 100.;
   distr = unur_distr_uniform(fparams,2);
   if (test_cdf_pdf( TESTLOG, distr,"t_distr_uniform.data" ) == 0)
      n_failed++;
   unur_distr_free(distr);

   fparams[0] = 10.;
   fparams[1] = 100.;
   distr = unur_distr_weibull(fparams,2);
   if (test_cdf_pdf( TESTLOG, distr,"t_distr_weibull.data" ) == 0)
      n_failed++;
   unur_distr_free(distr);

   fparams[0] = 1000.;
   fparams[1] = 0.999;
   distr = unur_distr_binomial(fparams,2);
   if (test_cdf_pdf( TESTLOG, distr,"t_distr_binomial.data" ) == 0)
      n_failed++;
   unur_distr_free(distr);

   fparams[0] = 0.999;
   distr = unur_distr_geometric(fparams,1);
   if (test_cdf_pdf( TESTLOG, distr,"t_distr_geometric.data" ) == 0)
      n_failed++;
   unur_distr_free(distr);

   fparams[0] = 1000.;
   fparams[1] = 99.;
   fparams[2] = 99.;
   distr = unur_distr_hypergeometric(fparams,3);
   if (test_cdf_pdf( TESTLOG, distr,"t_distr_hypergeometric.data" ) == 0)
      n_failed++;
   unur_distr_free(distr);

   fparams[0] = 0.999;
   distr = unur_distr_logarithmic(fparams,1);
   if (test_cdf_pdf( TESTLOG, distr,"t_distr_logarithmic.data" ) == 0)
      n_failed++;
   unur_distr_free(distr);

   fparams[0] = 0.999;
   fparams[1] = 100.;
   distr = unur_distr_negativebinomial(fparams,2);
   if (test_cdf_pdf( TESTLOG, distr,"t_distr_negativebinomial.data" ) == 0)
      n_failed++;
   unur_distr_free(distr);

   fparams[0] = 100.;
   distr = unur_distr_poisson(fparams,1);
   if (test_cdf_pdf( TESTLOG, distr,"t_distr_poisson.data" ) == 0)
      n_failed++;
   unur_distr_free(distr);

                                                                               
   /* close log file */                                                        
   fclose(UNURANLOG);                                                          
   fclose(TESTLOG);                                                            
                                                                               
   if (n_failed > 0)                                                           
      exit (-1);                                                               
   else                                                                        
      exit (0);                                                                
}                                                                              
