/*
	file automatically generated by make_test_files.pl
	Thu Apr  5 10:13:30 2001
*/

/*****************************************************************************
 *                                                                           *
 *          UNURAN -- Universal Non-Uniform Random number generator          *
 *                                                                           *
 *****************************************************************************/
    
/**
 ** Tests for DGT
 **/
    
/*---------------------------------------------------------------------------*/
#include "testunuran.h"
/*---------------------------------------------------------------------------*/
#ifdef T_DGT
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* global variables                                                          */

static struct prng *urng;           /* uniform random number generator       */

static FILE *TESTLOG;               /* test log file                         */
static FILE *UNURANLOG;             /* unuran log file                       */

static int test_ok = TRUE;          /* all tests ok (boolean)                */
static int n_tests_failed;          /* number of failed tests                */

/*---------------------------------------------------------------------------*/

void test_new( void );
void test_set( void );
void test_get( void );
void test_chg( void );
void test_init( void );
void test_reinit( void );
void test_sample( void );
void test_validate( void );

void run_verify_generator( FILE *LOG, int line, UNUR_PAR *par );

int unur_dgt_set_verify( UNUR_PAR *par, int );

/*---------------------------------------------------------------------------*/


/* prototypes */

static int COMPARE_SAMPLE_SIZE = 15000;
static int VIOLATE_SAMPLE_SIZE = 20;

double *make_geometric_vector(double q, int len);
double *make_random_vector(int len);
UNUR_DISTR *get_distr_with_invalid_pv( void );
UNUR_DISTR *get_distr_with_pv( void );





/*---------------------------------------------------------------------------*/

int main()
{ 
	/* open log file for unuran and set output stream for unuran messages */
	UNURANLOG = fopen( "t_dgt_unuran.log","w" );
	abort_if_NULL( stderr,-1, UNURANLOG );
	unur_set_stream( UNURANLOG );

	/* open log file for testing */
	TESTLOG = fopen( "t_dgt_test.log","w" );
	abort_if_NULL( stderr,-1, TESTLOG );

	/* write header into log file */
  	{
		time_t started;  
		fprintf(TESTLOG,"\nUNURAN - Universal Non-Uniform RANdom number generator\n\n");
		if (time( &started ) != -1)
			fprintf(TESTLOG,"%s",ctime(&started));
		fprintf(TESTLOG,"\n====================================================\n\n");
	}

	/* set uniform random number generator */
	urng = prng_new("mt19937(5678)");
	unur_set_default_urng(urng);

	/* set default debugging flag */
	unur_set_default_debug(UNUR_DEBUG_ALL);

	/* start test */
	printf("dgt: ");

	/* run tests */
	test_new();
	test_set();
	test_get();
	test_chg();
	test_init();
	test_sample();
	test_reinit();
	test_validate();

	/* test finished */
	printf("\n");  fflush(stdout);

	/* close log files */
	fprintf(TESTLOG,"\n====================================================\n\n");
	if (test_ok)
		fprintf(TESTLOG,"All tests PASSED.\n");
	else
		fprintf(TESTLOG,"Test(s) FAILED.\n");

	fclose(UNURANLOG);
	fclose(TESTLOG);

	/* exit */
	exit( (test_ok) ? 0 : -1 );

} /* end of main */

/*---------------------------------------------------------------------------*/

void test_new (void)
{
	/* start test */
	printf("[new "); fflush(stdout);
	fprintf(TESTLOG,"\n[new]\n");

	/* reset counter */
	n_tests_failed = 0;
  
/* invalid NULL ptr */
{
UNUR_DISTR *distr = NULL;
   distr = NULL; 

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 36, (unur_dgt_new( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 36, UNUR_ERR_NULL );
unur_distr_free(distr);
}

/* invalid distribution type */
{
UNUR_DISTR *distr = NULL;
   distr = unur_distr_cont_new(); 

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 42, (unur_dgt_new( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 42, UNUR_ERR_DISTR_INVALID );
unur_distr_free(distr);
}

/* data missing in distribution object */
{
UNUR_DISTR *distr = NULL;
    distr = unur_distr_discr_new();   /* no probability vector */ 

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 48, (unur_dgt_new( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 48, UNUR_ERR_DISTR_REQUIRED );
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = new  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_new() */

/*---------------------------------------------------------------------------*/

void test_set (void)
{
	/* start test */
	printf("[set "); fflush(stdout);
	fprintf(TESTLOG,"\n[set]\n");

	/* reset counter */
	n_tests_failed = 0;
  
/* invalid NULL ptr */
{
UNUR_PAR   *par = NULL;
   par = NULL; 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 58, (unur_dgt_set_variant(par,1)) );
n_tests_failed += check_errorcode( TESTLOG, 58, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 61, (unur_dgt_set_guidefactor(par,-1.)) );
n_tests_failed += check_errorcode( TESTLOG, 61, UNUR_ERR_NULL );
}

/* invalid parameter object */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
   distr = get_distr_with_pv();
   par = unur_dau_new(distr); 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 68, (unur_dgt_set_variant(par,10)) );
n_tests_failed += check_errorcode( TESTLOG, 68, UNUR_ERR_PAR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 71, (unur_dgt_set_guidefactor(par,-1.)) );
n_tests_failed += check_errorcode( TESTLOG, 71, UNUR_ERR_PAR_INVALID );
free(par);
unur_distr_free(distr);
}

/* invalid parameters */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
   distr = get_distr_with_pv();
   par = unur_dgt_new(distr); 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 80, (unur_dgt_set_variant(par,10)) );
n_tests_failed += check_errorcode( TESTLOG, 80, UNUR_ERR_PAR_VARIANT );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 83, (unur_dgt_set_guidefactor(par,-1.)) );
n_tests_failed += check_errorcode( TESTLOG, 83, UNUR_ERR_PAR_SET );
free(par);
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = set  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_set() */

/*---------------------------------------------------------------------------*/

void test_get (void)
{
	/* start test */
	printf("[get "); fflush(stdout);
	fprintf(TESTLOG,"\n[get]\n");

	/* reset counter */
	n_tests_failed = 0;
  

	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = get  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_get() */

/*---------------------------------------------------------------------------*/

void test_chg (void)
{
	/* start test */
	printf("[chg "); fflush(stdout);
	fprintf(TESTLOG,"\n[chg]\n");

	/* reset counter */
	n_tests_failed = 0;
  

	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = chg  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_chg() */

/*---------------------------------------------------------------------------*/

void test_init (void)
{
	/* start test */
	printf("[init "); fflush(stdout);
	fprintf(TESTLOG,"\n[init]\n");

	/* reset counter */
	n_tests_failed = 0;
  
/* invalid NULL ptr */
{
UNUR_PAR   *par = NULL;
   par = NULL; 

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 103, (unur_init( par )) );
n_tests_failed += check_errorcode( TESTLOG, 103, UNUR_ERR_NULL );
}

/* invalid data */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
   distr = get_distr_with_invalid_pv();
   par = unur_dgt_new(distr); 

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 111, (unur_init( par )) );
n_tests_failed += check_errorcode( TESTLOG, 111, UNUR_ERR_GEN_DATA );
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = init  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_init() */

/*---------------------------------------------------------------------------*/

void test_reinit (void)
{
	/* start test */
	printf("[reinit "); fflush(stdout);
	fprintf(TESTLOG,"\n[reinit]\n");

	/* reset counter */
	n_tests_failed = 0;
  

	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = reinit  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_reinit() */

/*---------------------------------------------------------------------------*/

void test_sample (void)
{
	/* start test */
	printf("[sample "); fflush(stdout);
	fprintf(TESTLOG,"\n[sample]\n");

	/* reset counter */
	n_tests_failed = 0;
  
/* compare */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
   distr = get_distr_with_pv();
   par = NULL; 

unur_errno = 0;
/* default algorithm */
par = unur_dgt_new(distr);
n_tests_failed += compare_int_sequence_par_start( TESTLOG, 127, urng, par, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
/* variant 1 */
par = unur_dgt_new(distr);
unur_dgt_set_variant(par,1);
n_tests_failed += compare_int_sequence_par( TESTLOG, 132, urng, par, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
/* variant 2 */
par = unur_dgt_new(distr);
unur_dgt_set_variant(par,2);
n_tests_failed += compare_int_sequence_par( TESTLOG, 137, urng, par, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
/* larger guide table size */
par = unur_dgt_new(distr);
unur_dgt_set_guidefactor(par,10);
n_tests_failed += compare_int_sequence_par( TESTLOG, 142, urng, par, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
/* smaller guide table size */
par = unur_dgt_new(distr);
unur_dgt_set_guidefactor(par,0.1);
n_tests_failed += compare_int_sequence_par( TESTLOG, 147, urng, par, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
/* sequential search */
par = unur_dgt_new(distr);
unur_dgt_set_guidefactor(par,0.);
n_tests_failed += compare_int_sequence_par( TESTLOG, 152, urng, par, COMPARE_SAMPLE_SIZE );
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = sample  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_sample() */

/*---------------------------------------------------------------------------*/

void test_validate (void)
{
	UNUR_DISTR *distr[22];
	UNUR_PAR *par;
	UNUR_GEN *gen;
	int n_tests_failed = 0;
	double *darray;
	double fpm[10];

	/* start test */
	printf("[validate "); fflush(stdout);
	fprintf(TESTLOG,"\n[validate]\n");

	/* reset counter */
	n_tests_failed = 0;


/* distributions: 22 */
distr[0] = unur_distr_discr_new();
darray = make_random_vector(10);
unur_distr_discr_set_pv(distr[0],darray,10);
free(darray);
unur_distr_set_name(distr[0],"pv(random)");


distr[1] = unur_distr_discr_new();
darray = make_random_vector(100);
unur_distr_discr_set_pv(distr[1],darray,100);
free(darray);
unur_distr_set_name(distr[1],"pv(random)");


distr[2] = unur_distr_discr_new();
darray = make_random_vector(1000);
unur_distr_discr_set_pv(distr[2],darray,1000);
free(darray);
unur_distr_set_name(distr[2],"pv(random)");


distr[3] = unur_distr_discr_new();
darray = make_random_vector(10000);
unur_distr_discr_set_pv(distr[3],darray,10000);
free(darray);
unur_distr_set_name(distr[3],"pv(random)");


distr[4] = unur_distr_discr_new();
darray = make_geometric_vector(1.,1000);
unur_distr_discr_set_pv(distr[4],darray,1000);
free(darray);
unur_distr_set_name(distr[4],"pv(geometric)");


distr[5] = unur_distr_discr_new();
darray = make_geometric_vector(0.99,1000);
unur_distr_discr_set_pv(distr[5],darray,1000);
free(darray);
unur_distr_set_name(distr[5],"pv(geometric)");


distr[6] = unur_distr_discr_new();
darray = make_geometric_vector(0.9,1000);
unur_distr_discr_set_pv(distr[6],darray,1000);
free(darray);
unur_distr_set_name(distr[6],"pv(geometric)");


distr[7] = unur_distr_discr_new();
darray = make_geometric_vector(0.5,1000);
unur_distr_discr_set_pv(distr[7],darray,1000);
free(darray);
unur_distr_set_name(distr[7],"pv(geometric)");


distr[8] = unur_distr_discr_new();
darray = make_geometric_vector(0.1,1000);
unur_distr_discr_set_pv(distr[8],darray,1000);
free(darray);
unur_distr_set_name(distr[8],"pv(geometric)");


distr[9] = unur_distr_discr_new();
darray = make_geometric_vector(0.5,100);
unur_distr_discr_set_pv(distr[9],darray,100);
unur_distr_discr_set_domain(distr[9],10,209);
free(darray);
unur_distr_set_name(distr[9],"pv(geometric) - shifted domain");


distr[10] = unur_distr_discr_new();
darray = make_geometric_vector(0.9,100);
unur_distr_discr_set_pv(distr[10],darray,100);
unur_distr_discr_set_domain(distr[10],-10,209);
free(darray);
unur_distr_set_name(distr[10],"pv(geometric) - shifted domain");


fpm[0] = 0.5;
distr[11] = unur_distr_geometric(fpm,1);


fpm[0] = 0.001;
distr[12] = unur_distr_geometric(fpm,1);


fpm[0] = 0.1;
distr[13] = unur_distr_logarithmic(fpm,1);


fpm[0] = 0.999;
distr[14] = unur_distr_logarithmic(fpm,1);


fpm[0] = 0.5;
fpm[1] = 10.;
distr[15] = unur_distr_negativebinomial(fpm,2);


fpm[0] = 0.8;
fpm[1] = 10.;
distr[21] = unur_distr_negativebinomial(fpm,2);


fpm[0] = 0.01;
fpm[1] = 20.;
distr[16] = unur_distr_negativebinomial(fpm,2);


fpm[0] = 0.1;
distr[17] = unur_distr_poisson(fpm,1);


fpm[0] = 0.999;
distr[18] = unur_distr_poisson(fpm,1);


fpm[0] = 2.;
fpm[1] = 1.;
distr[19] = unur_distr_zipf(fpm,2);
unur_distr_discr_set_domain(distr[19],1,1000);


fpm[0] = 0.001;
fpm[1] = 1.;
distr[20] = unur_distr_zipf(fpm,2);
unur_distr_discr_set_domain(distr[20],1,1000);


	printf("\n(chi^2) "); fflush(stdout);

/* chi^2 tests: 110 */

	unur_set_default_debug(~UNUR_DEBUG_SAMPLE);
	fprintf( TESTLOG,"\nChi^2 Test:\n");

/* distribution [0] */

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[0]);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[0]);
unur_dgt_set_variant(par,1);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[0]);
unur_dgt_set_variant(par,2);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[0]);
unur_dgt_set_guidefactor(par,10);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[0]);
unur_dgt_set_guidefactor(par,0.1);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [1] */

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[1]);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[1]);
unur_dgt_set_variant(par,1);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[1]);
unur_dgt_set_variant(par,2);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[1]);
unur_dgt_set_guidefactor(par,10);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[1]);
unur_dgt_set_guidefactor(par,0.1);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [2] */

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[2]);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[2]);
unur_dgt_set_variant(par,1);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[2]);
unur_dgt_set_variant(par,2);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[2]);
unur_dgt_set_guidefactor(par,10);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[2]);
unur_dgt_set_guidefactor(par,0.1);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [3] */

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[3]);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[3]);
unur_dgt_set_variant(par,1);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[3]);
unur_dgt_set_variant(par,2);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[3]);
unur_dgt_set_guidefactor(par,10);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[3]);
unur_dgt_set_guidefactor(par,0.1);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [4] */

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[4]);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[4]);
unur_dgt_set_variant(par,1);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[4]);
unur_dgt_set_variant(par,2);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[4]);
unur_dgt_set_guidefactor(par,10);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[4]);
unur_dgt_set_guidefactor(par,0.1);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [5] */

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[5]);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[5]);
unur_dgt_set_variant(par,1);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[5]);
unur_dgt_set_variant(par,2);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[5]);
unur_dgt_set_guidefactor(par,10);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[5]);
unur_dgt_set_guidefactor(par,0.1);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [6] */

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[6]);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[6]);
unur_dgt_set_variant(par,1);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[6]);
unur_dgt_set_variant(par,2);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[6]);
unur_dgt_set_guidefactor(par,10);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[6]);
unur_dgt_set_guidefactor(par,0.1);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [7] */

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[7]);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[7]);
unur_dgt_set_variant(par,1);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[7]);
unur_dgt_set_variant(par,2);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[7]);
unur_dgt_set_guidefactor(par,10);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[7]);
unur_dgt_set_guidefactor(par,0.1);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [8] */

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[8]);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[8]);
unur_dgt_set_variant(par,1);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[8]);
unur_dgt_set_variant(par,2);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[8]);
unur_dgt_set_guidefactor(par,10);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[8]);
unur_dgt_set_guidefactor(par,0.1);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [9] */

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[9]);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[9]);
unur_dgt_set_variant(par,1);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[9]);
unur_dgt_set_variant(par,2);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[9]);
unur_dgt_set_guidefactor(par,10);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[9]);
unur_dgt_set_guidefactor(par,0.1);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [10] */

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[10]);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[10]);
unur_dgt_set_variant(par,1);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[10]);
unur_dgt_set_variant(par,2);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[10]);
unur_dgt_set_guidefactor(par,10);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[10]);
unur_dgt_set_guidefactor(par,0.1);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [11] */

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[11]);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[11]);
unur_dgt_set_variant(par,1);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[11]);
unur_dgt_set_variant(par,2);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[11]);
unur_dgt_set_guidefactor(par,10);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[11]);
unur_dgt_set_guidefactor(par,0.1);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [12] */

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[12]);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[12]);
unur_dgt_set_variant(par,1);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[12]);
unur_dgt_set_variant(par,2);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[12]);
unur_dgt_set_guidefactor(par,10);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[12]);
unur_dgt_set_guidefactor(par,0.1);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [13] */

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[13]);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[13]);
unur_dgt_set_variant(par,1);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[13]);
unur_dgt_set_variant(par,2);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[13]);
unur_dgt_set_guidefactor(par,10);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[13]);
unur_dgt_set_guidefactor(par,0.1);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [14] */

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[14]);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[14]);
unur_dgt_set_variant(par,1);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[14]);
unur_dgt_set_variant(par,2);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[14]);
unur_dgt_set_guidefactor(par,10);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[14]);
unur_dgt_set_guidefactor(par,0.1);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [15] */

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[15]);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[15]);
unur_dgt_set_variant(par,1);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[15]);
unur_dgt_set_variant(par,2);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[15]);
unur_dgt_set_guidefactor(par,10);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[15]);
unur_dgt_set_guidefactor(par,0.1);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [21] */

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[21]);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[21]);
unur_dgt_set_variant(par,1);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[21]);
unur_dgt_set_variant(par,2);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[21]);
unur_dgt_set_guidefactor(par,10);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[21]);
unur_dgt_set_guidefactor(par,0.1);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [16] */

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[16]);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[16]);
unur_dgt_set_variant(par,1);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[16]);
unur_dgt_set_variant(par,2);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[16]);
unur_dgt_set_guidefactor(par,10);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[16]);
unur_dgt_set_guidefactor(par,0.1);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [17] */

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[17]);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[17]);
unur_dgt_set_variant(par,1);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[17]);
unur_dgt_set_variant(par,2);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[17]);
unur_dgt_set_guidefactor(par,10);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[17]);
unur_dgt_set_guidefactor(par,0.1);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [18] */

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[18]);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[18]);
unur_dgt_set_variant(par,1);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[18]);
unur_dgt_set_variant(par,2);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[18]);
unur_dgt_set_guidefactor(par,10);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[18]);
unur_dgt_set_guidefactor(par,0.1);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [19] */

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[19]);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[19]);
unur_dgt_set_variant(par,1);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[19]);
unur_dgt_set_variant(par,2);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[19]);
unur_dgt_set_guidefactor(par,10);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[19]);
unur_dgt_set_guidefactor(par,0.1);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [20] */

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[20]);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[20]);
unur_dgt_set_variant(par,1);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[20]);
unur_dgt_set_variant(par,2);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[20]);
unur_dgt_set_guidefactor(par,10);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dgt_new(distr[20]);
unur_dgt_set_guidefactor(par,0.1);
unur_set_debug(par, 1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);


/* free distributions */
	unur_distr_free(distr[0]);
	unur_distr_free(distr[1]);
	unur_distr_free(distr[2]);
	unur_distr_free(distr[3]);
	unur_distr_free(distr[4]);
	unur_distr_free(distr[5]);
	unur_distr_free(distr[6]);
	unur_distr_free(distr[7]);
	unur_distr_free(distr[8]);
	unur_distr_free(distr[9]);
	unur_distr_free(distr[10]);
	unur_distr_free(distr[11]);
	unur_distr_free(distr[12]);
	unur_distr_free(distr[13]);
	unur_distr_free(distr[14]);
	unur_distr_free(distr[15]);
	unur_distr_free(distr[16]);
	unur_distr_free(distr[17]);
	unur_distr_free(distr[18]);
	unur_distr_free(distr[19]);
	unur_distr_free(distr[20]);
	unur_distr_free(distr[21]);
#if WITH_DMALLOC
	dmalloc_vmessage("section = validate  ================================\n","");
	dmalloc_log_unfreed();
#endif


	/* test finished */
	if (n_tests_failed>0) n_tests_failed--;  /* we accept one failure */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf(" --> failed] ") : printf(" --> ok] ");

} /* end of test_validate */

/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/

/* make a probability vector (need not sum to one)
   (use random entries)                                                      */
double *make_random_vector(int len)
{
  double *pv;
  int i;

  /* allocate memory */
  pv = malloc(len*sizeof(double));
  if (!pv) abort();
     
  /* main part of geometric distribution */
  for( i=0; i<len; i++ ) 
    pv[i] = prng_get_next(urng);

  return pv;

} /* end of make_random_vector() */

/*---------------------------------------------------------------------------*/

/* make a probability vector (need not sum to one)
   (use geometric distribution)                                              */
double *make_geometric_vector(double q, int len)
{
  double *pv;
  int i;

  /* allocate memory */
  pv = malloc(len * sizeof(double));
  if (!pv) abort();

  /* main part of geometric distribution */
  pv[0] = 1.;
  for( i=1; i<len; i++ ) 
    pv[i] = pv[i-1] * q;

  return pv;

} /* end of make_geometric_vector() */

/*---------------------------------------------------------------------------*/

UNUR_DISTR *get_distr_with_pv( void )
{
#define PV_SIZE 200
  double pv[PV_SIZE];
  UNUR_DISTR *distr;
  int i;

  for (i=0; i<PV_SIZE; i++)
    pv[i] = prng_get_next(urng);

  distr = unur_distr_discr_new();
  unur_distr_discr_set_pv(distr,pv,PV_SIZE);

  return distr;

#undef PV_SIZE
} /* end of get_distr_with_pv() */

/*---------------------------------------------------------------------------*/

UNUR_DISTR *get_distr_with_invalid_pv( void )
{
#define PV_SIZE 10
  double pv[PV_SIZE];
  UNUR_DISTR *distr;
  int i;

  pv[0] = -1.;    /* invalid ! */

  for (i=1; i<PV_SIZE; i++)
    pv[i] = prng_get_next(urng);

  distr = unur_distr_discr_new();
  unur_distr_discr_set_pv(distr,pv,PV_SIZE);

  return distr;
  
#undef PV_SIZE
} /* end of get_distr_with_invalid_pv() */

/*---------------------------------------------------------------------------*/



/*---------------------------------------------------------------------------*/
/* run generator in verifying mode */

void run_verify_generator( FILE *LOG, int line, UNUR_PAR *par )
{
	UNUR_GEN *gen;
	int i;

	/* switch to verifying mode */
	unur_dgt_set_verify(par,1);

	/* initialize generator */
	gen = unur_init( par ); abort_if_NULL(LOG, line, gen);

	/* run generator */
	for (i=0; i<VIOLATE_SAMPLE_SIZE; i++)
		unur_sample_cont(gen);

	/* destroy generator */
	unur_free(gen); 

} /* end of run_verify_generator() */

int unur_dgt_set_verify(UNUR_PAR *par, int verify) {return 0;}
/*---------------------------------------------------------------------------*/
#else
/*---------------------------------------------------------------------------*/
int main() { exit(77); } /* ignore test */
/*---------------------------------------------------------------------------*/
#endif  /* T_DGT */
/*---------------------------------------------------------------------------*/
